package nl.tudelft.ewi.alg.distsim;

//  Copyright 2013 Léon Planken and James Boerkoel

/*
    This file is part of the DistSim package.

    DistSim is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DistSim is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DistSim.  If not, see <http://www.gnu.org/licenses/>.
*/

import nl.tudelft.ewi.alg.distsim.net.NetSimulator;

/**
 * The message class used internally by the package.
 * 
 * @param <M> The message payload (to be delivered to {@link DProcess}).
 *  
 * @author L&eacute;on Planken
 * @author James Boerkoel Jr.
 */

class Message<M> implements Comparable<Message<M>> {
	static final int UNUSED = Integer.MIN_VALUE;
	
	public final DProcess<M> sender;
	public final DProcess<M> receiver;
	public final M payload;
	public final int counter;
	public long deliveryTime;
	
	
	Message(DProcess<M> sender, DProcess<M> receiver, M payload, NetSimulator netsim) {
		this.sender   = sender;
		this.receiver = receiver;
		this.payload  = payload;
		this.counter  = UNUSED;
		
		if ( sender.id == receiver.id ) { //Shouldn't penalize self messages
		    this.deliveryTime = sender.getClock() + 1;
		} else {
		    this.deliveryTime = netsim.send(sender.id, receiver.id, sender.getClock());
		}
	}

	/**
	 * Construct a null message
	 * @param receiver The intended receiver
	 * @param deliveryTime The 
	 * @param counter
	 */
	Message(DProcess<M> receiver, long deliveryTime, int counter) {
		this.sender       = null;
		this.receiver     = receiver;
		this.payload      = null;
		this.counter      = counter;
		this.deliveryTime = deliveryTime;
	}

	public int compareTo(Message<M> other) {
		if ( this.deliveryTime > other.deliveryTime ) return  1;
		if ( this.deliveryTime < other.deliveryTime ) return -1;
		return 0;
	}
}
