package nl.tudelft.ewi.alg.distsim.example;

//  Copyright 2013 Léon Planken and James Boerkoel

/*
    This file is part of the DistSim package.

    DistSim is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DistSim is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DistSim.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.util.LinkedList;

/**
 * The message being passed around.
 * @author L&eacute;on Planken
 */
public class ChainLetter {
	/** Originator */
	public final int originatorID;
	/** Message text */
	public final String text;
	/** List of receiver IDs who should get this message (in order) */
	private LinkedList<Integer> receiverQueue;
	
	/**
	 * The  message being passed around.
	 * @param text Message text
	 * @param originatorID Originator of this chain letter
	 */
	public ChainLetter(String text, int originatorID) {
		this.originatorID = originatorID;
		this.text = text;
		this.receiverQueue = new LinkedList<Integer>();
	}

	public void enqueueReceiverID(int id) {
		this.receiverQueue.add(id);
	}

	public boolean queueEmpty() {
		return this.receiverQueue.isEmpty();
	}

	public int pollReceiverID() {
		return this.receiverQueue.poll();
	}

	public String queueContents() {
		return this.receiverQueue.toString();
	}
}
