package nl.tudelft.ewi.alg.distsim.example;

//  Copyright 2013 Léon Planken and James Boerkoel

/*
    This file is part of the DistSim package.

    DistSim is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DistSim is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DistSim.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.util.Random;

import nl.tudelft.ewi.alg.distsim.MasterProcess;
import nl.tudelft.ewi.alg.distsim.net.NetSimulator;
import nl.tudelft.ewi.alg.distsim.net.RandomDelay;

/**
 * An example distributed "algorithm" using the {@link nl.tudelft.ewi.alg.distsim} package.
 * A set of distributed processess send out some chain letters..
 * 
 * This class just sets everything up.  The actual algorithm is implemented in {@link MyProc}.
 * @author L&eacute;on Planken
 */
public class Example {
	public static final int N = 5;

	public static void main(String[] args) {
		System.out.println("This is a small working example for the DistSim package.");
		System.out.println("Version 1.0   Copyright 2013 Léon Planken and Jim Boerkoel");
		
		boolean assertionsOn = false;
		assert (assertionsOn = true);
		if ( !assertionsOn ) {
			System.out.println("Note: assertions are disabled.  Run with -ea flag to enable assertions.");
		}
		try {
			Thread.sleep(1000);
		} catch ( InterruptedException e ) { }
		
		System.out.println("");

		System.out.println("Creating MasterProcess with random network latency between 200 and 600 milliseconds");
		final NetSimulator netsim = new RandomDelay(new Random(), 200, 600);
		final MasterProcess<ChainLetter> master = new MasterProcess<ChainLetter>(netsim);
		final int[] processIDs = new int[N];
		final MyProc[] processes = new MyProc[N];

		for ( int i = 0; i < N; ++i ) {
			final MyProc p = new MyProc(master, i, N);
			System.out.println(String.format("Created new MyProc with id %d", p.id));
			processIDs[i] = p.id;
			processes[i] = p;
		}
		for ( int i = 0; i < N; ++i ) {
			processes[i].setProcs(processIDs);
		}
	
		System.out.println("Running algorithm...");
		master.run();
	}
}
