package nl.tudelft.ewi.alg.distsim.net;

//  Copyright 2013 Léon Planken and James Boerkoel

/*
    This file is part of the DistSim package.

    DistSim is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DistSim is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DistSim.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Abstract base class for implementing custom network simulators.
 * @author L&eacute;on Planken
 */

public interface NetSimulator {
	public final static long LOST = Long.MAX_VALUE;
	
	/**
	 * Simulate the sending of a message m by giving the message's delivery time, possibly losing it.
	 * @param senderID sender of the message
	 * @param receiverID receiver of the message
	 * @param sendTime clock time (in nanoseconds) when the message was sent
	 * @return clock time (in nanoseconds) when the message is to be delivered, or {@link NetSimulator#LOST} if lost
	 */
	public long send(int senderID, int receiverID, long sendTime);
}
