package nl.tudelft.ewi.alg.distsim.net;

//  Copyright 2013 Léon Planken and James Boerkoel

/*
    This file is part of the DistSim package.

    DistSim is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DistSim is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DistSim.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Simulator for a non-lossy network with a fixed message latency.
 * @author L&eacute;on Planken
 */

public class SimpleDelay implements NetSimulator {
	public final long delay;
	/**
	 * Units for specifying message latency.
	 * 
	 * @author L@eacute;on Planken
	 */
	public static enum Unit {
		/** nanoseconds */
		NS,
		/** microseconds */
		uS,
		/** milliseconds */
		MS,
		/** seconds */
		S,
	}
	
	/**
	 * Construct a SimpleDelay instance
	 * @param delay The (non-negative) delay for delivering messages, in milliseconds.
	 */
	public SimpleDelay(long delay) {
		this(delay,Unit.MS);
	}

	/**
	 * Construct a SimpleDelay instance
	 * @param delay The (non-negative) delay for delivering messages.
	 * @param unit the time unit in which the delay has been specified
	 */
	@SuppressWarnings("fallthrough")
	public SimpleDelay(long delay, Unit unit) {
		if ( delay < 0 ) throw new IllegalArgumentException("delay must be non-negative");
		
		switch(unit) {
		case S:
			delay *= 1000;
		case MS:
			delay *= 1000;
		case uS:
			delay *= 1000;
		case NS:
		}
		
		this.delay = delay;
	}

	/** {@inheritDoc} */
	public long send(int senderID, int receiverID, long sendTime) {
		return sendTime + delay;
	}
}
